@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import re
 import os
+import time
 import datetime
 import random
 from hdfs.client import Client
@@ -142,6 +143,13 @@ def random_timestamp(p_arg):
     return (random.random() * (etime - stime) + stime).strftime(frmt)
 
 
+# 返回系统当前时间,Unix时间戳方式
+def current_unixtimestamp(p_arg):
+    if p_arg:
+        pass
+    return str(int(time.mktime(datetime.datetime.now().timetuple())))
+
+
 # 返回系统当前时间
 def current_timestamp(p_arg):
     if len(p_arg) == 1:
@@ -153,25 +161,35 @@ def current_timestamp(p_arg):
     return datetime.datetime.now().strftime(frmt)
 
 
-# 第一个参数是seed的名字, 第二个参数是截取的最大长度
+# 第一个参数是seed的名字,
+# 第二个参数是开始截取的位置，可以省略。 如果指定，这里的开始位置从0开始计算
+# 第三个参数是截取的最大长度
 def random_from_seed(p_arg):
     m_SeedName = str(p_arg[0])
-    m_nMaxLength = int(p_arg[1])
+    if len(p_arg) == 3:
+        # 从指定位置开始截取数据内容
+        m_StartPos = int(p_arg[1])
+        m_nMaxLength = int(p_arg[2])
+    else:
+        # 从头开始截取文件内容
+        m_StartPos = 0
+        m_nMaxLength = int(p_arg[1])
 
-    # 如果还没有加载种子，就先加载
-    n = len(seed_cache)
-    if n == 0:
+    # 如果还没有加载种子，就先尝试加载
+    if m_SeedName not in seed_cache:
         Load_SeedCacheFile()
 
     # 在种子中查找需要的内容
     if m_SeedName in seed_cache:
         n = len(seed_cache[m_SeedName])
+        if n == 0:
+            # 空的Seed文件，可能是由于Seed文件刚刚创建，则尝试重新加载数据
+            Load_SeedCacheFile()
+            n = len(seed_cache[m_SeedName])
+            if n == 0:
+                raise SQLCliException("Seed cache is zero. [" + str(p_arg[0]) + "].")
         m_RandomRow = seed_cache[m_SeedName][random.randint(0, n - 1)]
-        if len(m_RandomRow) < m_nMaxLength:
-            m_RandomResult = m_RandomRow
-        else:
-            m_RandomResult = m_RandomRow[0:m_nMaxLength]
-        return m_RandomResult
+        return m_RandomRow[m_StartPos:m_StartPos+m_nMaxLength]
     else:
         raise SQLCliException("Unknown seed [" + str(p_arg[0]) + "].  Please create it first.")
 
@@ -227,57 +245,69 @@ def random_ascii_letters_and_digits(p_arg):
 
 
 # 返回一个自增长序列
-# 有1个参数：
-#   start   即自正常序列的开始值
+# 有2个参数：
+#   identity_name   自增序列号的名字
+#   start           即自正常序列的开始值
 #   步长目前没有支持，都是1
 def identity(p_arg):
-    nStart = int(p_arg[0])
+    identity_name = str(p_arg[0])
+    nStart = int(p_arg[1])
     if not hasattr(identity, 'x'):
-        identity.x = nStart
+        identity.x = {}
+    if identity_name not in identity.x.keys():
+        identity.x[identity_name] = nStart
     else:
-        identity.x = identity.x + 1
-    return str(identity.x)
+        identity.x[identity_name] = identity.x[identity_name] + 1
+    return str(identity.x[identity_name])
 
 
 # 返回一个自增长的时间戳
-# 有3个参数：
+# 有4个参数：
+#   identity_name   自增序列号的名字
 #   stime   开始时间
 #     若stime写作current_timestamp, 则自增上从当前时间开始
 #   frmt    日期格式，可以忽略，默认为%Y-%m-%d %H:%M:%S
 #   step    步长，可以用ms,us,s来表示，默认情况下是ms
 def identity_timestamp(p_arg):
-    if len(p_arg) == 2:
-        ptime = str(p_arg[0])
+    identity_name = str(p_arg[0])
+    if len(p_arg) == 3:
+        ptime = str(p_arg[1])
         frmt = "%Y-%m-%d %H:%M:%S"
-        step = str(p_arg[1])
-    elif len(p_arg) == 3:
-        ptime = str(p_arg[0])
-        frmt = str(p_arg[1])
+        step = str(p_arg[2])
+    elif len(p_arg) == 4:
+        ptime = str(p_arg[1])
+        frmt = str(p_arg[2])
         if len(frmt) == 0:
             frmt = "%Y-%m-%d %H:%M:%S"
-        step = str(p_arg[2])
+        step = str(p_arg[3])
     else:
         raise SQLCliException("Parameter error [" + str(p_arg[0]) + "].  Please use 2 or 3 parameters. double check it")
     if not hasattr(identity_timestamp, 'x'):
+        identity_timestamp.x = {}
+    if identity_name not in identity_timestamp.x.keys():
         if ptime == "current_timestamp":
-            identity_timestamp.x = datetime.datetime.now()
+            identity_timestamp.x[identity_name] = datetime.datetime.now()
         else:
-            identity_timestamp.x = datetime.datetime.strptime(ptime, frmt)
+            identity_timestamp.x[identity_name] = datetime.datetime.strptime(ptime, frmt)
     else:
         # 判断步长单位，默认是毫秒，可以是s,ms
         if step.endswith("s"):
             if step.endswith("ms"):
                 # end with ms 毫秒
-                identity_timestamp.x = (identity_timestamp.x + datetime.timedelta(milliseconds=float(step[:-2])))
+                identity_timestamp.x[identity_name] = (identity_timestamp.x[identity_name] +
+                                                       datetime.timedelta(milliseconds=float(step[:-2])))
             elif step.endswith("us"):
                 # end with us 微妙
-                identity_timestamp.x = (identity_timestamp.x + datetime.timedelta(microseconds=float(step[:-2])))
+                identity_timestamp.x[identity_name] = (identity_timestamp.x[identity_name] +
+                                                       datetime.timedelta(microseconds=float(step[:-2])))
             else:
                 # end with s 秒
-                identity_timestamp.x = (identity_timestamp.x + datetime.timedelta(seconds=float(step[:-1])))
+                identity_timestamp.x[identity_name] = (identity_timestamp.x[identity_name] +
+                                                       datetime.timedelta(seconds=float(step[:-1])))
         else:
-            identity_timestamp.x = (identity_timestamp.x + datetime.timedelta(milliseconds=float(step)))
-    return identity_timestamp.x.strftime(frmt)
+            identity_timestamp.x[identity_name] = (identity_timestamp.x[identity_name] +
+                                                   datetime.timedelta(milliseconds=float(step)))
+    return identity_timestamp.x[identity_name].strftime(frmt)
 
 
 # 将传递的SQL字符串转换成一个带有函数指针的数组
@@ -288,7 +318,8 @@ def parse_formula_str(p_formula_str):
     for m_nRowPos in range(0, len(m_row_struct)):
         if re.search('random_ascii_lowercase|random_ascii_uppercase|random_ascii_letters' +
                      '|random_digits|identity|identity_timestamp|random_ascii_letters_and_digits|random_from_seed' +
-                     '|random_date|random_timestamp|random_time|random_boolean|current_timestamp|value',
+                     '|random_date|random_timestamp|random_time|random_boolean|'
+                     'current_timestamp|current_unixtimestamp|value',
                      m_row_struct[m_nRowPos], re.IGNORECASE):
             m_function_struct = re.split(r'[(,)]', m_row_struct[m_nRowPos])
             for m_nPos in range(0, len(m_function_struct)):
@@ -316,7 +347,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_ascii_lowercase)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -336,7 +367,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_ascii_uppercase)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -356,7 +387,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_ascii_letters)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -376,13 +407,15 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_digits)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
             elif m_function_struct[0].upper() == "IDENTITY_TIMESTAMP":
                 m_call_out_struct.append(identity_timestamp)
-                m_call_out_struct.append(m_function_struct[1:])
+                # 如果没有列名，第一个参数信息是identity#加上列号的信息
+                m_function_struct[0] = "identity#" + str(m_nRowPos)
+                m_call_out_struct.append(m_function_struct)
                 m_call_out_struct.append("__NO_NAME__")
             elif re.search(r"(.*):IDENTITY_TIMESTAMP", m_function_struct[0].upper()):
                 matchObj = re.search(r"(.*):IDENTITY_TIMESTAMP", m_function_struct[0].upper())
@@ -396,13 +429,17 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(identity_timestamp)
-                m_call_out_struct.append(m_function_struct[1:])
+                # 如果有列名，第一个参数信息是identity#加上列名的信息
+                m_function_struct[0] = "identity#" + m_ColumnName
+                m_call_out_struct.append(m_function_struct)
                 m_call_out_struct.append(m_ColumnName)
             elif m_function_struct[0].upper() == "IDENTITY":
                 m_call_out_struct.append(identity)
-                m_call_out_struct.append(m_function_struct[1:])
+                # 如果没有列名，第一个参数信息是identity#加上列号的信息
+                m_function_struct[0] = "identity#" + str(m_nRowPos)
+                m_call_out_struct.append(m_function_struct)
                 m_call_out_struct.append("__NO_NAME__")
             elif re.search(r"(.*):IDENTITY", m_function_struct[0].upper()):
                 matchObj = re.search(r"(.*):IDENTITY", m_function_struct[0].upper())
@@ -416,9 +453,11 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(identity)
-                m_call_out_struct.append(m_function_struct[1:])
+                # 如果有列名，第一个参数信息是identity#加上列名的信息
+                m_function_struct[0] = "identity#" + m_ColumnName
+                m_call_out_struct.append(m_function_struct)
                 m_call_out_struct.append(m_ColumnName)
             elif m_function_struct[0].upper() == "RANDOM_ASCII_LETTERS_AND_DIGITS":
                 m_call_out_struct.append(random_ascii_letters_and_digits)
@@ -436,7 +475,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_ascii_letters_and_digits)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -456,7 +495,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_from_seed)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -476,7 +515,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_date)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -496,7 +535,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_timestamp)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -516,7 +555,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_time)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -536,7 +575,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(random_boolean)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
@@ -556,10 +595,30 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 m_call_out_struct.append(current_timestamp)
                 m_call_out_struct.append(m_function_struct[1:])
                 m_call_out_struct.append(m_ColumnName)
+            elif m_function_struct[0].upper() == "CURRENT_UNIXTIMESTAMP":
+                m_call_out_struct.append(current_unixtimestamp)
+                m_call_out_struct.append(m_function_struct[1:])
+                m_call_out_struct.append("__NO_NAME__")
+            elif re.search(r"(.*):CURRENT_UNIXTIMESTAMP", m_function_struct[0].upper()):
+                matchObj = re.search(r"(.*):CURRENT_UNIXTIMESTAMP", m_function_struct[0].upper())
+                m_ColumnName = matchObj.group(1).upper().strip()
+                # 检查列名是否已经定义
+                bFound = False
+                for row in m_return_row_struct:
+                    if isinstance(row, list):
+                        if row[2] == m_ColumnName:
+                            bFound = True
+                            break
+                if bFound:
+                    raise SQLCliException("Invalid pattern. "
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
+                m_call_out_struct.append(current_unixtimestamp)
+                m_call_out_struct.append(m_function_struct[1:])
+                m_call_out_struct.append(m_ColumnName)
             elif m_function_struct[0].upper() == "VALUE":
                 # 必须用：开头来表示字段名称
                 if not m_function_struct[1:][0].startswith(":"):
@@ -579,7 +638,7 @@ def parse_formula_str(p_formula_str):
                         if isinstance(row, list):
                             m_ValidColumn = m_ValidColumn + row[2] + "|"
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is valid.\n" +
+                                          "Please make sure column [" + m_ColumnName + "] is valid.\n" +
                                           "Valid Column=[" + m_ValidColumn + "]")
                 m_call_out_struct.append(None)
                 m_call_out_struct.append(m_ColumnName)
@@ -596,7 +655,7 @@ def parse_formula_str(p_formula_str):
                             break
                 if bFound:
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is not duplicate.")
+                                          "Please make sure column [" + m_ColumnName + "] is not duplicate.")
                 # 必须用：开头来表示字段名称
                 if not m_function_struct[1:][0].startswith(":"):
                     raise SQLCliException("Invalid pattern. Please use Value(:ColumnName).")
@@ -615,7 +674,7 @@ def parse_formula_str(p_formula_str):
                         if isinstance(row, list):
                             m_ValidColumn = m_ValidColumn + row[2] + "|"
                     raise SQLCliException("Invalid pattern. "
-                                          "Please make sure columename [" + m_ColumnName + "] is valid.\n" +
+                                          "Please make sure column [" + m_ColumnName + "] is valid.\n" +
                                           "Valid Column=[" + m_ValidColumn + "]")
                 m_call_out_struct.append(None)
                 m_call_out_struct.append(m_ColumnName)
@@ -640,7 +699,7 @@ def get_final_string(p_row_struct):
             elif col[2] == "__NO_NAME__":
                 m_Result = m_Result + col[0](col[1])
             else:
-                m_Value = col[0](col[1])
+                m_Value = col[0](col[1])              # 根据函数指针计算返回后的实际内容
                 m_Saved_ColumnData[col[2]] = m_Value
                 m_Result = m_Result + m_Value
         else:
